@@ -18,19 +18,28 @@
 #ifndef OPKG_CMD_H
 #define OPKG_CMD_H
 
-typedef int (*opkg_cmd_fun_t)(opkg_conf_t *conf, int argc, const char **argv);
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+typedef int (*opkg_cmd_fun_t)(int argc, const char **argv);
 
 struct opkg_cmd
 {
-    char *name;
+    const char *name;
     int requires_args;
     opkg_cmd_fun_t fun;
+    unsigned int pfm; /* package field mask */
 };
 typedef struct opkg_cmd opkg_cmd_t;
 
 opkg_cmd_t *opkg_cmd_find(const char *name);
-int opkg_cmd_exec(opkg_cmd_t *cmd, opkg_conf_t *conf, int argc, 
-                  const char **argv, void *userdata);
+int opkg_cmd_exec(opkg_cmd_t *cmd, int argc, const char **argv);
 
 extern int opkg_state_changed;
+
+#ifdef __cplusplus
+}
 #endif
+
+#endif /* OPKG_CMD_H */
