@@ -18,8 +18,16 @@
 #ifndef OPKG_CONFIGURE_H
 #define OPKG_CONFIGURE_H
 
-#include "opkg_conf.h"
-
-int opkg_configure(opkg_conf_t *opkg_conf, pkg_t *pkg);
-
+#ifdef __cplusplus
+extern "C" {
 #endif
+
+#include "pkg.h"
+
+int opkg_configure(pkg_t *pkg);
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* OPKG_CONFIGURE_H */
