@@ -358,7 +358,7 @@ pkg_remove_orphan_dependent(pkg_t *pkg, pkg_t *old_pkg)
 
 	for (i=0; i<count0; i++) {
 		cd0 = &old_pkg->depends[i];
-		if (cd0->type != DEPEND)
+		if (cd0->type != DEPEND && cd0->type != RECOMMEND)
 			continue;
 		for (j=0; j<cd0->possibility_count; j++) {
 
@@ -366,11 +366,10 @@ pkg_remove_orphan_dependent(pkg_t *pkg, pkg_t *old_pkg)
 
 			for (k=0; k<count1; k++) {
 				cd1 = &pkg->depends[k];
-				if (cd1->type != DEPEND)
+				if (cd1->type != DEPEND && cd1->type != RECOMMEND)
 					continue;
 				for (l=0; l<cd1->possibility_count; l++) {
-					if (cd0->possibilities[j]
-					 == cd1->possibilities[l]) {
+					if (cd0->possibilities[j]->pkg == cd1->possibilities[l]->pkg) {
 						found = 1;
 						break;
 					}
@@ -1626,6 +1625,18 @@ opkg_install_pkg(pkg_t *pkg, int from_upgrade)
 	  pkg_remove_installed_replacees_unwind(replacees);
 
 pkg_is_hosed:
+	  /* Set the package flags to something consistent which indicates a
+	   * failed install.
+	   */
+	  pkg->state_flag = SF_REINSTREQ;
+	  pkg->state_status = SS_HALF_INSTALLED;
+
+	  /* Print some advice for the user. */
+	  opkg_msg(NOTICE, "To remove package debris, try `opkg remove %s`.\n",
+			  pkg->name);
+	  opkg_msg(NOTICE, "To re-attempt the install, try `opkg install %s`.\n",
+			  pkg->name);
+
 	  sigprocmask(SIG_UNBLOCK, &newset, &oldset);
 
           pkg_vec_free (replacees);
