@@ -58,10 +58,10 @@ free_pkgs(const char *key, void *entry, void *data)
 		}
 	}
 
+	abstract_pkg_vec_free (ab_pkg->depended_upon_by);
 	abstract_pkg_vec_free (ab_pkg->provided_by);
 	abstract_pkg_vec_free (ab_pkg->replaced_by);
 	pkg_vec_free (ab_pkg->pkgs);
-	free (ab_pkg->depended_upon_by);
 	free (ab_pkg->name);
 	free (ab_pkg);
 }
@@ -272,7 +272,6 @@ pkg_hash_fetch_best_installation_candidate(abstract_pkg_t *apkg,
 {
      int i, j;
      int nprovides = 0;
-     int nmatching = 0;
      pkg_vec_t *matching_pkgs;
      abstract_pkg_vec_t *matching_apkgs;
      abstract_pkg_vec_t *provided_apkg_vec;
@@ -354,7 +353,7 @@ pkg_hash_fetch_best_installation_candidate(abstract_pkg_t *apkg,
                * on a later version of the held package.
                */
               if ((maybe->state_status == SS_INSTALLED || maybe->state_status == SS_UNPACKED)
-                      && maybe->state_flag == SF_HOLD)
+                      && (maybe->state_flag & SF_HOLD))
                   goto add_matching_pkg;
 
               /* Ensure that the package meets the specified constraint. */
@@ -476,8 +475,6 @@ add_matching_pkg:
 	  }
      }
 
-     nmatching = matching_apkgs->len;
-
      pkg_vec_free(matching_pkgs);
      abstract_pkg_vec_free(matching_apkgs);
      abstract_pkg_vec_free(providers);
@@ -500,17 +497,13 @@ add_matching_pkg:
 			priorized_matching->architecture);
 	  return priorized_matching;
      }
-     if (nmatching > 1) {
-	  opkg_msg(INFO, "No matching pkg out of %d matching_apkgs.\n",
-			nmatching);
-	  return NULL;
-     }
      if (latest_matching) {
 	  opkg_msg(INFO, "Using latest matching %s %s %s.\n",
 			latest_matching->name, latest_matching->version,
 			latest_matching->architecture);
 	  return latest_matching;
      }
+     opkg_msg(INFO, "No matching pkg for '%s'.\n", apkg->name);
      return NULL;
 }
 
