@@ -50,6 +50,7 @@ enum {
 	ARGS_OPT_NODEPS,
 	ARGS_OPT_AUTOREMOVE,
 	ARGS_OPT_CACHE,
+        ARGS_OPT_COMBINE,
 };
 
 static struct option long_options[] = {
@@ -58,6 +59,7 @@ static struct option long_options[] = {
 	{"cache", 1, 0, ARGS_OPT_CACHE},
 	{"conf-file", 1, 0, 'f'},
 	{"conf", 1, 0, 'f'},
+        {"combine", 0, 0, ARGS_OPT_COMBINE},
 	{"dest", 1, 0, 'd'},
         {"force-maintainer", 0, 0, ARGS_OPT_FORCE_MAINTAINER},
         {"force_maintainer", 0, 0, ARGS_OPT_FORCE_MAINTAINER},
@@ -202,6 +204,9 @@ args_parse(int argc, char *argv[])
         case ARGS_OPT_DOWNLOAD_ONLY:
 			conf->download_only = 1;
 			break;
+                case ARGS_OPT_COMBINE:
+                        conf->combine = 1;
+                        break;
 		case ':':
 			parse_err = -1;
 			break;
@@ -226,83 +231,85 @@ usage()
 	printf("where sub-command is one of:\n");
 
 	printf("\nPackage Manipulation:\n");
-	printf("\tupdate			Update list of available packages\n");
-	printf("\tupgrade			Upgrade installed packages\n");
-	printf("\tinstall <pkgs>		Install package(s)\n");
-	printf("\tconfigure <pkgs>	Configure unpacked package(s)\n");
-	printf("\tremove <pkgs|regexp>	Remove package(s)\n");
-	printf("\tflag <flag> <pkgs>	Flag package(s)\n");
+	printf("\tupdate                          Update list of available packages\n");
+	printf("\tupgrade                         Upgrade installed packages\n");
+	printf("\tinstall <pkgs>                  Install package(s)\n");
+	printf("\tconfigure <pkgs>                Configure unpacked package(s)\n");
+	printf("\tremove <pkgs|glob>              Remove package(s)\n");
+	printf("\tflag <flag> <pkgs>              Flag package(s)\n");
 	printf("\t <flag>=hold|noprune|user|ok|installed|unpacked (one per invocation)\n");
 
 	printf("\nInformational Commands:\n");
-	printf("\tlist			List available packages\n");
-	printf("\tlist-installed		List installed packages\n");
-	printf("\tlist-upgradable		List installed and upgradable packages\n");
-	printf("\tlist-changed-conffiles	List user modified configuration files\n");
-	printf("\tfiles <pkg>		List files belonging to <pkg>\n");
-	printf("\tsearch <file|regexp>	List package providing <file>\n");
-	printf("\tinfo [pkg|regexp]	Display all info for <pkg>\n");
-	printf("\tstatus [pkg|regexp]	Display all status for <pkg>\n");
-	printf("\tdownload <pkg>		Download <pkg> to current directory\n");
+	printf("\tlist                            List available packages\n");
+	printf("\tlist-installed                  List installed packages\n");
+	printf("\tlist-upgradable                 List installed and upgradable packages\n");
+	printf("\tlist-changed-conffiles          List user modified configuration files\n");
+	printf("\tfiles <pkg>                     List files belonging to <pkg>\n");
+	printf("\tsearch <file|glob>              List package providing <file>\n");
+	printf("\tinfo [pkg|glob]                 Display all info for <pkg>\n");
+	printf("\tstatus [pkg|glob]               Display all status for <pkg>\n");
+	printf("\tdownload <pkg>                  Download <pkg> to current directory\n");
 	printf("\tcompare-versions <v1> <op> <v2>\n");
-	printf("\t                    compare versions using <= < > >= = << >>\n");
-	printf("\tprint-architecture	List installable package architectures\n");
-	printf("\tdepends [-A] [pkgname|pat]+\n");
-	printf("\twhatdepends [-A] [pkgname|pat]+\n");
-	printf("\twhatdependsrec [-A] [pkgname|pat]+\n");
-	printf("\twhatrecommends[-A] [pkgname|pat]+\n");
-	printf("\twhatsuggests[-A] [pkgname|pat]+\n");
-	printf("\twhatprovides [-A] [pkgname|pat]+\n");
-	printf("\twhatconflicts [-A] [pkgname|pat]+\n");
-	printf("\twhatreplaces [-A] [pkgname|pat]+\n");
+	printf("\t                                compare versions using <= < > >= = << >>\n");
+	printf("\tprint-architecture              List installable package architectures\n");
+	printf("\tdepends [-A] [pkgname|glob]+\n");
+	printf("\twhatdepends [-A] [pkgname|glob]+\n");
+	printf("\twhatdependsrec [-A] [pkgname|glob]+\n");
+	printf("\twhatrecommends[-A] [pkgname|glob]+\n");
+	printf("\twhatsuggests[-A] [pkgname|glob]+\n");
+	printf("\twhatprovides [-A] [pkgname|glob]+\n");
+	printf("\twhatconflicts [-A] [pkgname|glob]+\n");
+	printf("\twhatreplaces [-A] [pkgname|glob]+\n");
 
 	printf("\nOptions:\n");
-	printf("\t-A			Query all packages not just those installed\n");
-	printf("\t-V[<level>]		Set verbosity level to <level>.\n");
-	printf("\t--verbosity[=<level>]	Verbosity levels:\n");
-	printf("\t				0 errors only\n");
-	printf("\t				1 normal messages (default)\n");
-	printf("\t				2 informative messages\n");
-	printf("\t				3 debug\n");
-	printf("\t				4 debug level 2\n");
-	printf("\t-f <conf_file>		Use <conf_file> as the opkg configuration file\n");
+	printf("\t-A                              Query all packages not just those installed\n");
+	printf("\t-V[<level>]                     Set verbosity level to <level>.\n");
+	printf("\t--verbosity[=<level>]           Verbosity levels:\n");
+	printf("\t                                  0 errors only\n");
+	printf("\t                                  1 normal messages (default)\n");
+	printf("\t                                  2 informative messages\n");
+	printf("\t                                  3 debug\n");
+	printf("\t                                  4 debug level 2\n");
+	printf("\t-f <conf_file>                  Use <conf_file> as the opkg configuration file\n");
 	printf("\t--conf <conf_file>\n");
-	printf("\t--cache <directory>	Use a package cache\n");
-	printf("\t-d <dest_name>		Use <dest_name> as the the root directory for\n");
-	printf("\t--dest <dest_name>	package installation, removal, upgrading.\n");
-	printf("				<dest_name> should be a defined dest name from\n");
-	printf("				the configuration file, (but can also be a\n");
-	printf("				directory name in a pinch).\n");
-	printf("\t-o <dir>		Use <dir> as the root directory for\n");
-	printf("\t--offline-root <dir>	offline installation of packages.\n");
-	printf("\t--add-arch <arch>:<prio>	Register architecture with given priority\n");
-	printf("\t--add-dest <name>:<path>	Register destination with given path\n");
-	printf("\t--prefer-arch-to-version\t 	Use the architecture priority package rather\n");
-	printf("\t				than the higher version one if more\n");
-	printf("\t				than one candidate is found.\n");
+	printf("\t--cache <directory>             Use a package cache\n");
+	printf("\t-d <dest_name>                  Use <dest_name> as the the root directory for\n");
+	printf("\t--dest <dest_name>              package installation, removal, upgrading.\n");
+	printf("\t                                <dest_name> should be a defined dest name from\n");
+	printf("\t                                the configuration file, (but can also be a\n");
+	printf("\t                                directory name in a pinch).\n");
+	printf("\t-o <dir>                        Use <dir> as the root directory for\n");
+	printf("\t--offline-root <dir>            offline installation of packages.\n");
+	printf("\t--add-arch <arch>:<prio>        Register architecture with given priority\n");
+	printf("\t--add-dest <name>:<path>        Register destination with given path\n");
+	printf("\t--prefer-arch-to-version        Use the architecture priority package rather\n");
+	printf("\t                                than the higher version one if more\n");
+	printf("\t                                than one candidate is found.\n");
+        printf("\t--combine                       Combine upgrade and install operations, this\n");
+        printf("\t                                may be needed to resolve dependency issues.\n");
 
 	printf("\nForce Options:\n");
-	printf("\t--force-depends		Install/remove despite failed dependencies\n");
-	printf("\t--force-maintainer	Overwrite preexisting config files\n");
-	printf("\t--force-reinstall	Reinstall package(s)\n");
-	printf("\t--force-overwrite	Overwrite files from other package(s)\n");
-	printf("\t--force-downgrade	Allow opkg to downgrade packages\n");
-	printf("\t--force-space		Disable free space checks\n");
-	printf("\t--force-postinstall	Run postinstall scripts even in offline mode\n");
-	printf("\t--force-remove	Remove package even if prerm script fails\n");
-	printf("\t--noaction		No action -- test only\n");
-	printf("\t--download-only	No action -- download only\n");
-	printf("\t--nodeps		Do not follow dependencies\n");
+	printf("\t--force-depends                 Install/remove despite failed dependencies\n");
+	printf("\t--force-maintainer              Overwrite preexisting config files\n");
+	printf("\t--force-reinstall               Reinstall package(s)\n");
+	printf("\t--force-overwrite               Overwrite files from other package(s)\n");
+	printf("\t--force-downgrade               Allow opkg to downgrade packages\n");
+	printf("\t--force-space                   Disable free space checks\n");
+	printf("\t--force-postinstall             Run postinstall scripts even in offline mode\n");
+	printf("\t--force-remove                  Remove package even if prerm script fails\n");
+	printf("\t--noaction                      No action -- test only\n");
+	printf("\t--download-only                 No action -- download only\n");
+	printf("\t--nodeps                        Do not follow dependencies\n");
 	printf("\t--force-removal-of-dependent-packages\n");
-	printf("\t			Remove package and all dependencies\n");
-	printf("\t--autoremove		Remove packages that were installed\n");
-	printf("\t			automatically to satisfy dependencies\n");
-	printf("\t-t			Specify tmp-dir.\n");
-	printf("\t--tmp-dir		Specify tmp-dir.\n");
+	printf("\t                                Remove package and all dependencies\n");
+	printf("\t--autoremove                    Remove packages that were installed\n");
+	printf("\t                                automatically to satisfy dependencies\n");
+	printf("\t-t                              Specify tmp-dir.\n");
+	printf("\t--tmp-dir                       Specify tmp-dir.\n");
 
 	printf("\n");
 
-	printf(" regexp could be something like 'pkgname*' '*file*' or similar\n");
+	printf(" glob could be something like 'pkgname*' '*file*' or similar\n");
 	printf(" e.g. opkg info 'libstd*' or opkg search '*libop*' or opkg remove 'libncur*'\n");
 
 	/* --force-removal-of-essential-packages	Let opkg remove essential packages.
